// map
.map-container {
    height: 400px;
}


.map-wrap {
    position: relative;
    width: 100%;
    height: calc(100vh - 200px);
}

.map {
    position: absolute;
    width: 100%;
    height: 100%;
}


.sidebar {
    background-color: rgba(35, 55, 75, 0.9);
    color: #fff;
    padding: 6px 12px;
    z-index: 1;
    position: absolute;
    top: 0;
    left: 0;
    margin: 12px;
    border-radius: 4px;
    font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, "Noto Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
    font-size: 14px;
    word-spacing: 5px;
}

.maplibregl-ctrl-attrib-inner {
    font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, "Noto Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
}

.select-metric {
    width: 160px;
}


/*popup*/
.mbview-popup {
    color: #333;
    display: table;
    font-size: 10px;
}

.maplibregl-popup-content {
    border-radius: 10px;
    box-shadow: 4px 1px 56px -8px rgba(0,0,0,0.75);
    font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, "Noto Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
}



.mbview_feature:not(:last-child) {
    border-bottom: 1px solid #ccc;
    padding: 6px 2px;
}

.mbview_layer:before {
    content: '§ ';
}

.mbview_layer {
    display: block;
    font-weight: bold;
}
.mbview_property {
    display: table-row;
}
.mbview_property-value {
    display: table-cell;
}
.mbview_property-name {
    display: table-cell;
    padding-right: 10px;
}