@import "@cloudscape-design/global-styles/index.css";


.logo {
  float: left;
  width: 150px;
  height: 32px;
  margin: 16px 24px 16px 0;
  object-fit:cover;
}


.ant-row-rtl .logo {
  float: right;
  margin: 16px 0 16px 24px;
}

.site-layout-background {
  background: #fff;
}

